package com.javatunes.persist.client;

import java.math.BigDecimal;
import java.util.List;

import javax.persistence.EntityManager;
import javax.persistence.EntityManagerFactory;
import javax.persistence.EntityTransaction;
import javax.persistence.Persistence;
import javax.persistence.TypedQuery;

import com.javatunes.persist.MusicItem;

public class JPATest {
	public static void main(String[] args) {
		EntityManagerFactory emf = Persistence.createEntityManagerFactory("javatunes"); 
		EntityManager em = emf.createEntityManager();
		EntityTransaction tx = null;
		try {
			
			// Simple query - no parameters
			tx = em.getTransaction();
			tx.begin();
		    TypedQuery<MusicItem> q = em.createQuery("SELECT m FROM MusicItem m", MusicItem.class);
		    List<MusicItem> resultList = q.getResultList();
		    System.out.println("***** Results from retrieving all music items *********");
		    for (MusicItem curItem : resultList) {
		    	System.out.println(curItem);
		    }
			tx.commit();			

			// Query with parameters
			tx = em.getTransaction();
			tx.begin();
			String artistQuery = "SELECT mi FROM MusicItem mi WHERE mi.artist = :artist";
			// Positional parameter version
			// String keyQuery = "SELECT mi FROM MusicItem mi WHERE mi.artist = ?";
		    q = em.createQuery(artistQuery, MusicItem.class);
		    q.setParameter("artist", "Sting");
			// Positional parameter version
		    // q.setParameter(1, "Sting");
		    resultList = q.getResultList();
		    System.out.println("***** Results from artist query on Sting *********");
		    for (MusicItem curItem : resultList) {
		    	System.out.println(curItem);
		    }
		    q.setParameter("artist", "Madonna");
		    resultList = q.getResultList();
		    System.out.println("***** Results from artist query on Madonna *********");
		    for (MusicItem curItem : resultList) {
		    	System.out.println(curItem);
		    }
		    tx.commit();			

			// Named Query
			tx = em.getTransaction();
			tx.begin();
			// Positional parameter version
			// String keyQuery = "SELECT mi FROM MusicItem mi WHERE mi.artist = ?";
		    q = em.createNamedQuery("MusicItem.findByArtist", MusicItem.class);
		    q.setParameter("artist", "Sting");
			// Positional parameter version
		    // q.setParameter(1, "Sting");
		    resultList = q.getResultList();
		    System.out.println("***** Results from named query on Sting *********");
		    for (MusicItem curItem : resultList) {
		    	System.out.println(curItem);
		    }
		    q.setParameter("artist", "Madonna");
		    resultList = q.getResultList();
		    System.out.println("***** Results from named query on Madonna *********");
		    for (MusicItem curItem : resultList) {
		    	System.out.println(curItem);
		    }
		    tx.commit();			

		    artistQuery = "SELECT mi FROM MusicItem mi WHERE mi.artist = :artist AND mi.price < :price";
			// Positional parameter version
			// String keyQuery = "SELECT mi FROM MusicItem mi WHERE mi.artist = ?";
		    q = em.createQuery(artistQuery, MusicItem.class);
		    q.setParameter("artist", "Sting");
		    q.setParameter("price", new BigDecimal("11.00"));
			// Positional parameter version
		    // q.setParameter(1, "Madonna");
		    resultList = q.getResultList();
		    System.out.println("***** Results from artist query on Sting *********");
		    for (MusicItem curItem : resultList) {
		    	System.out.println(curItem);
		    }
		    
			
		} 
		catch (Exception ex) { ex.printStackTrace(); }
		finally { em.close(); }
	}
}
