package com.javatunes.persist.client;

import java.math.BigDecimal;
import java.util.List;

import javax.persistence.EntityManager;
import javax.persistence.EntityManagerFactory;
import javax.persistence.EntityTransaction;
import javax.persistence.Persistence;
import javax.persistence.Query;
import javax.persistence.TypedQuery;

import com.javatunes.persist.PriceInfo;

public class JPATest {
	public static void main(String[] args) {
		EntityManagerFactory emf = Persistence.createEntityManagerFactory("javatunes"); 
		EntityManager em = emf.createEntityManager();
		EntityTransaction tx = null;
		try {
			tx = em.getTransaction();
			tx.begin();

		   // Simple Projection query 
		   String projQuery = "SELECT i.id, i.title FROM MusicItem i";
		   Query q = em.createQuery(projQuery);
		   List<Object[]> results = (List<Object[]>)q.getResultList();
		   System.out.println("***** Results from retrieving projection query *********");
		   for (Object[] cur : results) {
			   System.out.println(cur[0] + " : " + cur[1]);
		   }

		  // Projection query using Java object
		   String priceInfoQuery = "SELECT new com.javatunes.persist.PriceInfo (mi.listPrice, mi.price) FROM MusicItem mi";
		   Query q2 = em.createQuery(priceInfoQuery);
		   List<PriceInfo> priceInfoResults = (List<PriceInfo>)q2.getResultList();
		   System.out.println("***** Results from PriceInfo query *********");
		   for (PriceInfo cur : priceInfoResults) {
			   System.out.println(cur);
		   }
			   
		   // Aggregate query
		   String agQuery = "SELECT max(i.listPrice) FROM MusicItem i";
		   TypedQuery<BigDecimal> q3 = em.createQuery(agQuery, BigDecimal.class);
		   BigDecimal agResults = q3.getSingleResult();
		   System.out.println("***** Results from aggregate query *********");
		   System.out.println(agResults);
		   
		   // Optional - GROUP BY
		   String groupQuery = "SELECT i.artist, avg(i.price) FROM MusicItem i GROUP BY i.artist";
		   Query q4 = em.createQuery(groupQuery);
		   List<Object[]> groupResults = (List<Object[]>)q4.getResultList();
		   System.out.println("***** Results from GROUP BY query *********");
		   for (Object[] cur : groupResults) {
			   System.out.println(cur[0] + " : " + cur[1]);
		   }		   
		   
		
		    tx.commit();				
		} 
		catch (Exception ex) { ex.printStackTrace(); }
		finally { em.close(); }
	}
}
