package com.javatunes.persist.client;


import java.util.Collection;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;

import javax.persistence.EntityManager;
import javax.persistence.EntityManagerFactory;
import javax.persistence.Persistence;
import javax.persistence.TypedQuery;

import com.javatunes.persist.Inventory;
import com.javatunes.persist.MusicItem;

public class JPATest {
	public static void main(String[] args) {
		EntityManagerFactory emf = Persistence.createEntityManagerFactory("javatunes"); 
		EntityManager em = emf.createEntityManager();
		try {
// ****  Code to be run without cascading set
/*
			// Creating a new inventory record 
			em.getTransaction().begin();
			MusicItem mi = em.find(MusicItem.class, new Long (5));
			mi.addInventoryRecord("Los Angeles", 22);
			em.getTransaction().commit();
			
			// Modifying a managed inventory record
			em.getTransaction().begin();
			mi = em.find(MusicItem.class, new Long (5));
			Collection<Inventory> invRecords = mi.getInventoryRecords();
			for (Inventory cur : invRecords) {
				cur.setQuantity(cur.getQuantity() + 100);
			}
			em.getTransaction().commit();
*/

// **** Code to be run with and without orphan removal
/*

			// Removing an inventory record
			em.getTransaction().begin();
			MusicItem mi2 = em.find(MusicItem.class, new Long (1));
			System.out.println("Inventory before removal");
			for (Inventory cur : mi2.getInventoryRecords()) {
				System.out.println(cur);
			}
			
			Collection<Inventory> invRecords2 = mi2.getInventoryRecords();
			Iterator<Inventory> i = invRecords2.iterator();
			Inventory anInventory = i.next();
			invRecords2.remove(anInventory);
			em.getTransaction().commit();
*/
			
/*			
			// Lazy fetch 
			em.getTransaction().begin();
			MusicItem mi3 = em.find(MusicItem.class, new Long (1));
			for (Inventory cur : mi3.getInventoryRecords()) {
				System.out.println(cur);
			}			
			em.getTransaction().commit();
*/

/* 
			// Lazy fetch with detached object issue
			em.getTransaction().begin();
			MusicItem mi3 = em.find(MusicItem.class, new Long (1));
			em.getTransaction().commit();
			em.close();
			
			for (Inventory cur : mi3.getInventoryRecords()) {
				System.out.println(cur);
			}
*/
/*			
			// Eager fetch
			em.getTransaction().begin();
			String queryString = "SELECT m FROM MusicItem m LEFT JOIN FETCH m.inventoryRecords WHERE m.id = :id";
			TypedQuery<MusicItem> tq = em.createQuery(queryString, MusicItem.class);
			tq.setParameter("id", new Long(1));
			Collection<MusicItem> items = tq.getResultList();
			System.out.println("Number of items retrieved from query = " + items.size());
			HashSet<MusicItem> filteredItems = new HashSet<MusicItem>(items);
			System.out.println("Number of items after filtering = " + filteredItems.size());
			MusicItem theItem = filteredItems.iterator().next();
			for (Inventory cur : theItem.getInventoryRecords()) {
				System.out.println(cur);
			}			
			em.getTransaction().commit();
*/
/*
			// Eager fetch with detached objects
			em.getTransaction().begin();
			String queryString = "SELECT m FROM MusicItem m LEFT JOIN FETCH m.inventoryRecords WHERE m.id = :id";
			TypedQuery<MusicItem> tq = em.createQuery(queryString, MusicItem.class);
			tq.setParameter("id", new Long(1));
			Collection<MusicItem> items = tq.getResultList();
			System.out.println("Number of items retrieved from query = " + items.size());
			HashSet<MusicItem> filteredItems = new HashSet<MusicItem>(items);
			System.out.println("Number of items after filtering = " + filteredItems.size());
			MusicItem theItem = filteredItems.iterator().next();
			em.getTransaction().commit();
			em.close();
			for (Inventory cur : theItem.getInventoryRecords()) {
				System.out.println(cur);
			}
*/
			// Query over a relationship
			em.getTransaction().begin();
			String locationQueryString = "SELECT i.location FROM MusicItem mi JOIN mi.inventoryRecords i where mi.id = :id"; 
			TypedQuery<String> locationQuery = em.createQuery(locationQueryString, String.class);
			locationQuery.setParameter("id", new Long(1));
			for (String curLocation : locationQuery.getResultList()) {
				System.out.println(curLocation);
			}
			em.getTransaction().commit();
		} 
		catch (Exception ex) { ex.printStackTrace(); }
		finally { em.close(); }
	}
}
