package com.javatunes.persist.client;

import java.math.BigDecimal;
import java.util.List;

import javax.persistence.EntityManager;
import javax.persistence.EntityManagerFactory;
import javax.persistence.Persistence;
import javax.persistence.TypedQuery;
import javax.persistence.criteria.CriteriaBuilder;
import javax.persistence.criteria.CriteriaQuery;
import javax.persistence.criteria.Join;
import javax.persistence.criteria.JoinType;
import javax.persistence.criteria.Predicate;
import javax.persistence.criteria.Root;

import com.javatunes.persist.Inventory;
import com.javatunes.persist.MusicItem;

public class JPATest {
	public static void main(String[] args) {
		EntityManagerFactory emf = Persistence.createEntityManagerFactory("javatunes"); 
		EntityManager em = emf.createEntityManager();
		try {
			
			// Criteria query retrieving all MusicItems
			em.getTransaction().begin();
			CriteriaBuilder cb = em.getCriteriaBuilder();

			CriteriaQuery<MusicItem> cq1 = cb.createQuery(MusicItem.class);
			Root<MusicItem> root1 = cq1.from(MusicItem.class);
		
			cq1.select(root1);
			TypedQuery<MusicItem> tq = em.createQuery(cq1);
			List<MusicItem> results1 = tq.getResultList();

			System.out.println("\n\nFound " + results1.size() + " items via criteria query");
			for (MusicItem cur : results1) {
				System.out.println(cur);
			}

			// Same select, adding condition where id=1
			cq1.select(root1).where(cb.equal(root1.<Long>get("id"), new Long(1) ));
			tq = em.createQuery(cq1);
			results1 = tq.getResultList();
			System.out.println("\n\nFound " + results1.size() + " items with id = 1 via criteria query");
			for (MusicItem cur : results1) {
				System.out.println(cur);
			}
					
			// Select titles (only) for all MusicItems.
			CriteriaQuery<String> cq2 = cb.createQuery(String.class);
			Root<MusicItem> root2 = cq2.from(MusicItem.class);
			cq2.select(root2.<String>get("title"));
			TypedQuery<String> tq2 = em.createQuery(cq2);
			List<String> results2 = tq2.getResultList();
			System.out.println("\n\nFound " + results2.size() + " titles");
			for (String cur : results2) {
				System.out.println(cur);
			}
			
			// Select MusicItems with price<14.00 and id>10
			CriteriaQuery<MusicItem> cq3 = cb.createQuery(MusicItem.class);
			Root<MusicItem> root3 = cq3.from(MusicItem.class);
			Predicate criteria = cb.conjunction();
			criteria = cb.and(criteria, cb.lessThan(root3.<BigDecimal>get("price"), new BigDecimal("14.00")));
			criteria = cb.and(criteria, cb.greaterThan(root3.<Long>get("id"), new Long(10))); 
			cq3.select(root3).where(criteria);
			TypedQuery<MusicItem> tq3 = em.createQuery(cq3);
			List<MusicItem> results3 = tq3.getResultList();
			System.out.println("\n\nFound " + results3.size() + " items with price<13 and id>10");
			for (MusicItem cur : results3) {
				System.out.println(cur);
			}
			
			// Equivalent of "SELECT i from MusicItem mi JOIN mi.inventoryRecords i WHERE mi.id=1"
			CriteriaQuery<Inventory> cq4 = cb.createQuery(Inventory.class);
			Root<MusicItem> root4 = cq4.from(MusicItem.class);
			Join<MusicItem, Inventory> join4 = root4.join("inventoryRecords");
			cq4.select(join4).where(cb.equal(root4.<Long>get("id"), new Long(1)));
			TypedQuery<Inventory> tq4 = em.createQuery(cq4);
			List<Inventory> results4 = tq4.getResultList();
			System.out.println("\n\nFound " + results4.size() + " inventory via criteria query");
			for (Inventory cur : results4) {
				System.out.println(cur);
			}

			// Eager fetch of inventory records
			System.out.println("\n\n\n");
			System.out.println("Eager Fetch of Inventory Records");
			CriteriaQuery<MusicItem> cq5 = cb.createQuery(MusicItem.class);
			Root<MusicItem> root5 = cq5.from(MusicItem.class);
			root5.fetch("inventoryRecords", JoinType.LEFT);
			cq5.select(root5);
			TypedQuery<MusicItem> tq5 = em.createQuery(cq5);
			List<MusicItem> results5 = tq5.getResultList();
			for (MusicItem cur : results5) {
				System.out.println(cur.getId() + " : " + cur.getInventoryRecords().size());
			}			
			em.getTransaction().commit();

		} 
		catch (Exception ex) { ex.printStackTrace(); }
		finally { em.close(); }
	}
}
