/*
 * This code is sample code, provided as-is, and we make no
 * warranties as to its correctness or suitablity for
 * any purpose.
 *
 * We hope that it's useful to you.  Enjoy.
 * Copyright 2006-7 LearningPatterns Inc.
 */

package com.javatunes.jpa.client;

import com.javatunes.jpa.catalog.MusicItem;
import com.javatunes.jpa.catalog.MusicItemServiceRemote;

import javax.naming.*;
import javax.rmi.PortableRemoteObject;

public class MusicItemServiceClient
{
   public static void main(String[] args)
   {
      // declare initial context outside of try block so we can reference it in a finally block
      InitialContext ctx = null;

      try
      {
         // InitialContext is the starting point for JNDI name lookup
         // initial context factory and provider url are supplied via system properties
         ctx = new InitialContext();

         // look up the bean's remote business interface in JNDI
         Object lookup = ctx.lookup("catalog/MusicItemServiceBean/remote");

         // cast lookup object to Catalog
         // this type of cast is required to be compatible with systems that use RMI-IIOP
         // the narrow method must throw an exception or return an object that can be safely cast
         MusicItemServiceRemote miService = (MusicItemServiceRemote) PortableRemoteObject.narrow(lookup, MusicItemServiceRemote.class);


	     // Example of looking up by ID.
	     MusicItem item1 = miService.findById(new Long(1));
	     System.out.println("*** Looking up by ID=1 ***");
	     System.out.println("Item 1 = " + item1);
             System.out.println("****");
             System.out.println();


	     // Adding an item (very simplisticly)
	     MusicItem item2 = new MusicItem();
	     item2.setTitle("A new release!");
	     item2.setArtist("Bob Dylan");
	     item2.setNum("CDNNN");
	     Long newId = miService.addItem(item2);
	     System.out.println("Added new item with id = " + newId);
             System.out.println("****");
             System.out.println();

             // Modifying the title of an item
             Long modifyId = new Long(3);
             miService.updateTitle(modifyId, "A New nifty Title");
	     // Look up modified item and print out title
	     MusicItem modifiedItem = miService.findById(modifyId);
	     System.out.println("*** Looking up modified item with ID = " + modifyId + " ***");
	     System.out.println("Modified item title = " + modifiedItem.getTitle());
	     System.out.println();		 
      }
      catch (NamingException e)
      {
         System.out.println("JNDI exception");
         e.printStackTrace();
      }
      finally
      {
         // release JNDI resources when done with them
         try
         {
            // close JNDI initial context
            if (ctx != null)
            {
               ctx.close();
            }
         }
         catch (NamingException ignored) { }
      }
   }
}
